type token =
  | IDENT of (string)
  | STRING of (string)
  | NUM of (string)
  | BANG_
  | PERCENT_
  | AMPER_
  | AMPER_AMPER_
  | LPAREN_
  | LPAREN_RPAREN_
  | RPAREN_
  | STAR_
  | PLUS_
  | PLUS_PLUS_
  | COMMA_
  | DASH_
  | DASH_GT_
  | PERIOD_
  | PERIOD_LPAREN_
  | PERIOD_PERIOD_
  | SLASH_
  | SLASH_BACKSL_
  | COLON_
  | COLON_COLON_
  | COLON_LT_
  | COLON_EQ_
  | COLON_GT_
  | SEMI_
  | LT_
  | LT_DASH_
  | LT_DASH_GT_
  | LT_COLON_
  | LT_EQ_
  | LT_GT_
  | EQ_
  | EQ_GT_
  | EQ_UNDER_D_
  | GT_
  | GT_DASH_GT_
  | GT_EQ_
  | QUEST_
  | QUEST_EQ_
  | AROBAS_
  | LBRACK_
  | BACKSL_SLASH_
  | RBRACK_
  | HAT_
  | LBRACE_
  | BAR_
  | BAR_DASH_
  | BAR_BAR_
  | RBRACE_
  | TILDE_
  | MUSTUSE
  | DEFINITION
  | DEPENDS
  | ELSE
  | END
  | EXISTS
  | FALSE
  | FIX
  | FIXPOINT
  | FORALL
  | FUN
  | FUNCTION
  | IF
  | IN
  | INDUCTIVE
  | LET
  | MATCH
  | ON
  | PARAMETER
  | STRUCT
  | THEN
  | THEOREM
  | TRUE
  | WITH
  | BEGINPROOF
  | BEGINNAME of (string)
  | BEGINHEADER
  | ENDPROOF
  | EOF

open Parsing;;
let _ = parse_error;;
# 4 "parsecoq.mly"
Version.add "$Id: parsecoq.mly,v 1.34 2012-04-11 18:27:26 doligez Exp $";;

open Printf;;

open Expr;;
open Namespace;;
open Phrase;;

let rec mk_type_string e =
  match e with
  | Evar (s, _) -> s
  | Emeta _ -> assert false
  | Eapp ("*", [e1; e2], _) ->
     sprintf "(%s*%s)" (mk_type_string e1) (mk_type_string e2)
  | Eapp ("%", [e1; e2], _) ->
     sprintf "((%s)%%%s)" (mk_type_string e1) (mk_type_string e2)
  | Eapp (s, args, _) ->
     let inside =
       List.fold_left (fun s a -> sprintf "%s %s" s (mk_type_string a)) s args
     in
     sprintf "(%s)" inside
  | Eimply (e1, e2, _) ->
     sprintf "(%s -> %s)" (mk_type_string e1) (mk_type_string e2)
  | _ -> assert false (* FIXME TODO *)
;;

let mk_eapp (s, args) =
  match (s, args) with
  | "and", [e1; e2] -> eand (e1, e2)
  | "or", [e1; e2] -> eor (e1, e2)
  | "not", [e1] -> enot (e1)
  | _ -> eapp (s, args)
;;

let mk_apply (e, l) =
  match e with
  | Eapp (s, args, _) -> mk_eapp (s, args @ l)
  | Evar (s, _) -> mk_eapp (s, l)
  | _ -> raise Parse_error
;;

let rec mk_arobas_apply (id, l) =
  match l with
  | Evar ("_", _) :: t -> mk_arobas_apply (id, t)
  | [] -> evar (id)
  | _ -> eapp (id, l)
;;

let mk_all bindings body =
  let f (var, ty) e = eall (evar var, ty, e) in
  List.fold_right f bindings body
;;

let mk_ex bindings body =
  let f (var, ty) e = eex (evar var, ty, e) in
  List.fold_right f bindings body
;;

let mk_lam bindings body =
  let f (var, ty) e = elam (evar var, ty, e) in
  List.fold_right f bindings body
;;

let mk_fix ident ty bindings body =
  let f (var, ty) e = elam (evar var, ty, e) in
  (ident, eapp ("$fix", [ List.fold_right f ((ident, ty) :: bindings) body ]))
;;

let rec get_params e =
  match e with
  | Elam (v, _, body, _) ->
      let (p, e1) = get_params body in
      (v :: p, e1)
  | _ -> ([], e)
;;

let mk_let id expr body =
  substitute_2nd [(evar id, expr)] body
;;

let mk_let_fix (id, def) body = mk_let id def body;;

let mk_pattern (constr, args) body =
  let bindings = List.map (fun v -> (v, "")) args in
  mk_lam bindings (eapp ("$match-case", [evar (constr); body]))
;;

let mk_inductive name bindings constrs =
  let args = List.map fst bindings in
  let g (tcon, targs) =
    if tcon = name && targs = args then Self
    else Param (String.concat " " (tcon :: targs))
  in
  let f (cname, args) = (cname, List.map g args) in
  Inductive (name, args, List.map f constrs, name ^ "_ind")
;;

let mk_pairs e l =
  let f x y = eapp ("@", [evar "Datatypes.pair"; evar "_"; evar "_"; x; y]) in
  List.fold_left f e l
;;

let mk_string s = evar ("\"" ^ s ^ "\"") ;;
# 191 "parsecoq.ml"
let yytransl_const = [|
  260 (* BANG_ *);
  261 (* PERCENT_ *);
  262 (* AMPER_ *);
  263 (* AMPER_AMPER_ *);
  264 (* LPAREN_ *);
  265 (* LPAREN_RPAREN_ *);
  266 (* RPAREN_ *);
  267 (* STAR_ *);
  268 (* PLUS_ *);
  269 (* PLUS_PLUS_ *);
  270 (* COMMA_ *);
  271 (* DASH_ *);
  272 (* DASH_GT_ *);
  273 (* PERIOD_ *);
  274 (* PERIOD_LPAREN_ *);
  275 (* PERIOD_PERIOD_ *);
  276 (* SLASH_ *);
  277 (* SLASH_BACKSL_ *);
  278 (* COLON_ *);
  279 (* COLON_COLON_ *);
  280 (* COLON_LT_ *);
  281 (* COLON_EQ_ *);
  282 (* COLON_GT_ *);
  283 (* SEMI_ *);
  284 (* LT_ *);
  285 (* LT_DASH_ *);
  286 (* LT_DASH_GT_ *);
  287 (* LT_COLON_ *);
  288 (* LT_EQ_ *);
  289 (* LT_GT_ *);
  290 (* EQ_ *);
  291 (* EQ_GT_ *);
  292 (* EQ_UNDER_D_ *);
  293 (* GT_ *);
  294 (* GT_DASH_GT_ *);
  295 (* GT_EQ_ *);
  296 (* QUEST_ *);
  297 (* QUEST_EQ_ *);
  298 (* AROBAS_ *);
  299 (* LBRACK_ *);
  300 (* BACKSL_SLASH_ *);
  301 (* RBRACK_ *);
  302 (* HAT_ *);
  303 (* LBRACE_ *);
  304 (* BAR_ *);
  305 (* BAR_DASH_ *);
  306 (* BAR_BAR_ *);
  307 (* RBRACE_ *);
  308 (* TILDE_ *);
  309 (* MUSTUSE *);
  310 (* DEFINITION *);
  311 (* DEPENDS *);
  312 (* ELSE *);
  313 (* END *);
  314 (* EXISTS *);
  315 (* FALSE *);
  316 (* FIX *);
  317 (* FIXPOINT *);
  318 (* FORALL *);
  319 (* FUN *);
  320 (* FUNCTION *);
  321 (* IF *);
  322 (* IN *);
  323 (* INDUCTIVE *);
  324 (* LET *);
  325 (* MATCH *);
  326 (* ON *);
  327 (* PARAMETER *);
  328 (* STRUCT *);
  329 (* THEN *);
  330 (* THEOREM *);
  331 (* TRUE *);
  332 (* WITH *);
  333 (* BEGINPROOF *);
  335 (* BEGINHEADER *);
  336 (* ENDPROOF *);
    0 (* EOF *);
    0|]

let yytransl_block = [|
  257 (* IDENT *);
  258 (* STRING *);
  259 (* NUM *);
  334 (* BEGINNAME *);
    0|]

let yylhs = "\255\255\
\001\000\001\000\001\000\001\000\004\000\004\000\005\000\005\000\
\003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
\003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\
\003\000\003\000\003\000\003\000\007\000\010\000\010\000\010\000\
\010\000\010\000\010\000\010\000\010\000\010\000\011\000\011\000\
\013\000\013\000\009\000\009\000\014\000\014\000\014\000\006\000\
\006\000\016\000\015\000\015\000\017\000\017\000\017\000\012\000\
\018\000\018\000\019\000\019\000\019\000\019\000\019\000\019\000\
\020\000\020\000\022\000\022\000\022\000\002\000\002\000\021\000\
\021\000\023\000\023\000\023\000\024\000\024\000\008\000\008\000\
\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\
\000\000"

let yylen = "\002\000\
\007\000\009\000\003\000\005\000\004\000\002\000\000\000\002\000\
\004\000\004\000\004\000\004\000\006\000\010\000\008\000\005\000\
\006\000\003\000\003\000\003\000\003\000\003\000\003\000\002\000\
\002\000\003\000\002\000\001\000\011\000\001\000\001\000\001\000\
\004\000\005\000\005\000\003\000\001\000\001\000\001\000\002\000\
\002\000\003\000\000\000\005\000\003\000\002\000\005\000\001\000\
\001\000\003\000\000\000\002\000\000\000\002\000\004\000\001\000\
\001\000\001\000\005\000\005\000\008\000\012\000\011\000\008\000\
\000\000\006\000\003\000\002\000\001\000\002\000\000\000\005\000\
\000\000\001\000\003\000\005\000\003\000\002\000\000\000\002\000\
\002\000\002\000\002\000\002\000\002\000\002\000\004\000\004\000\
\002\000"

let yydefred = "\000\000\
\000\000\000\000\030\000\032\000\031\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\038\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\037\000\000\000\
\089\000\000\000\000\000\000\000\000\000\069\000\000\000\000\000\
\000\000\024\000\068\000\000\000\058\000\000\000\000\000\000\000\
\000\000\000\000\000\000\048\000\049\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\057\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\025\000\
\070\000\000\000\036\000\000\000\000\000\000\000\026\000\000\000\
\000\000\000\000\067\000\052\000\054\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\008\000\
\000\000\000\000\000\000\000\000\000\000\023\000\022\000\000\000\
\003\000\000\000\000\000\040\000\000\000\000\000\000\000\033\000\
\000\000\000\000\000\000\000\000\000\000\000\000\050\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\005\000\000\000\
\000\000\000\000\035\000\034\000\042\000\000\000\000\000\060\000\
\055\000\000\000\000\000\000\000\000\000\080\000\083\000\082\000\
\000\000\084\000\081\000\085\000\000\000\086\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\016\000\059\000\000\000\
\000\000\004\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\046\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\088\000\087\000\000\000\000\000\000\000\000\000\
\045\000\000\000\001\000\000\000\066\000\061\000\000\000\000\000\
\000\000\064\000\000\000\000\000\000\000\000\000\044\000\000\000\
\000\000\000\000\000\000\000\000\000\000\047\000\002\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\063\000\078\000\000\000\000\000\072\000\000\000\000\000\062\000\
\000\000\077\000\000\000\075\000\000\000\000\000\076\000"

let yydgoto = "\002\000\
\025\000\026\000\126\000\028\000\060\000\042\000\055\000\143\000\
\149\000\029\000\072\000\127\000\078\000\181\000\043\000\044\000\
\045\000\038\000\030\000\081\000\210\000\031\000\244\000\245\000"

let yysindex = "\070\000\
\081\003\000\000\000\000\000\000\000\000\166\255\103\255\166\255\
\176\255\151\255\029\255\101\255\000\000\109\255\101\255\101\255\
\113\255\166\255\115\255\001\255\166\255\204\255\000\000\071\255\
\000\000\088\255\251\003\195\003\023\255\000\000\239\001\049\001\
\023\255\000\000\000\000\156\255\000\000\161\255\176\255\101\255\
\180\255\169\255\181\255\000\000\000\000\156\255\196\255\182\255\
\156\255\179\255\112\255\022\255\210\255\213\255\153\255\163\255\
\000\000\198\255\071\255\144\255\225\255\166\255\166\255\166\255\
\166\255\166\255\166\255\246\000\174\255\251\003\023\255\000\000\
\000\000\248\255\000\000\166\255\166\255\240\255\000\000\250\255\
\231\255\166\255\000\000\000\000\000\000\180\255\244\255\166\255\
\166\255\209\255\166\255\166\255\237\255\166\255\112\255\238\255\
\127\001\166\255\239\255\101\255\166\255\217\255\166\255\000\000\
\071\255\247\255\211\255\125\255\211\255\000\000\000\000\085\255\
\000\000\007\000\186\255\000\000\004\000\191\000\143\001\000\000\
\255\255\166\255\249\000\112\255\241\000\241\000\000\000\207\255\
\241\000\241\000\166\255\242\255\023\000\127\001\127\001\127\001\
\127\001\127\001\127\001\127\001\127\001\127\001\003\000\155\255\
\127\001\234\255\241\000\164\255\226\255\057\001\000\000\166\255\
\015\000\029\001\000\000\000\000\000\000\166\255\014\000\000\000\
\000\000\031\000\249\255\166\255\018\000\000\000\000\000\000\000\
\034\000\000\000\000\000\000\000\036\000\000\000\166\255\166\255\
\043\000\243\255\180\255\172\255\028\000\000\000\000\000\173\001\
\166\255\000\000\050\000\166\255\020\000\166\255\241\000\030\000\
\127\001\127\001\017\000\241\000\041\000\081\000\000\000\049\255\
\076\000\166\255\087\001\231\001\156\255\091\002\066\000\183\000\
\089\000\078\000\000\000\000\000\166\255\166\255\046\000\097\000\
\000\000\061\002\000\000\021\000\000\000\000\000\166\255\077\000\
\082\000\000\000\241\000\075\000\095\000\090\000\000\000\111\001\
\093\000\166\255\190\255\166\255\166\255\000\000\000\000\166\255\
\149\002\180\255\193\255\030\000\105\000\241\000\099\000\207\002\
\000\000\000\000\193\255\011\255\000\000\190\255\166\255\000\000\
\119\000\000\000\190\255\000\000\241\000\126\000\000\000"

let yyrindex = "\000\000\
\071\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\063\255\000\000\000\000\063\255\089\255\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\073\255\
\000\000\000\000\140\001\071\000\001\000\000\000\005\000\000\000\
\059\000\000\000\000\000\048\255\000\000\000\000\000\000\108\255\
\124\000\000\000\000\000\000\000\000\000\101\000\000\000\000\000\
\131\000\000\000\132\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\073\255\118\003\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\079\000\117\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\127\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\135\255\000\000\
\130\000\000\000\000\000\250\254\000\000\103\000\000\000\000\000\
\160\003\000\000\041\001\175\000\099\001\000\000\000\000\233\000\
\000\000\000\000\000\000\000\000\000\000\000\000\146\000\000\000\
\000\000\000\000\000\000\135\255\157\001\098\255\000\000\000\000\
\215\001\017\002\000\000\000\000\000\000\004\255\004\255\004\255\
\148\000\004\255\004\255\004\255\148\000\004\255\000\000\000\000\
\148\000\000\000\075\002\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\128\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\133\002\149\000\
\004\255\004\255\000\000\191\002\000\000\000\000\000\000\157\000\
\000\000\000\000\000\000\000\000\040\255\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\103\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\249\002\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\038\255\000\000\149\000\041\255\051\003\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\110\000\000\000\000\000"

let yygindex = "\000\000\
\000\000\025\000\002\000\000\000\208\255\253\255\000\000\154\255\
\207\000\245\255\012\000\140\255\058\001\255\000\216\255\141\001\
\233\255\165\001\010\000\214\255\206\000\000\000\158\255\093\255"

let yytablesize = 1346
let yytable = "\084\000\
\028\000\052\000\027\000\090\000\071\000\159\000\093\000\032\000\
\053\000\034\000\104\000\047\000\048\000\079\000\163\000\051\000\
\085\000\071\000\035\000\050\000\002\001\071\000\056\000\003\000\
\004\000\005\000\003\001\096\000\079\000\070\000\006\000\166\000\
\167\000\168\000\169\000\170\000\171\000\172\000\173\000\174\000\
\053\000\187\000\177\000\097\000\079\000\084\000\098\000\051\000\
\083\000\086\000\074\000\068\000\069\000\051\000\051\000\073\000\
\151\000\074\000\027\000\071\000\054\000\065\000\216\000\107\000\
\108\000\109\000\110\000\111\000\112\000\065\000\001\000\085\000\
\057\000\007\000\007\000\007\000\053\000\118\000\119\000\252\000\
\007\000\013\000\116\000\123\000\051\000\051\000\065\000\001\001\
\074\000\125\000\211\000\212\000\129\000\130\000\115\000\132\000\
\146\000\023\000\039\000\144\000\161\000\040\000\147\000\033\000\
\150\000\063\000\233\000\056\000\041\000\046\000\051\000\056\000\
\095\000\049\000\007\000\051\000\039\000\065\000\066\000\041\000\
\247\000\053\000\056\000\053\000\007\000\007\000\007\000\007\000\
\067\000\051\000\007\000\007\000\056\000\007\000\007\000\007\000\
\007\000\007\000\199\000\007\000\007\000\007\000\053\000\007\000\
\056\000\063\000\007\000\007\000\053\000\059\000\007\000\036\000\
\037\000\184\000\053\000\004\001\053\000\065\000\066\000\199\000\
\006\001\061\000\221\000\080\000\179\000\191\000\003\000\004\000\
\005\000\053\000\062\000\180\000\200\000\006\000\021\000\063\000\
\195\000\196\000\062\000\180\000\086\000\053\000\088\000\063\000\
\064\000\082\000\204\000\065\000\066\000\206\000\242\000\208\000\
\064\000\242\000\062\000\065\000\066\000\243\000\067\000\063\000\
\251\000\250\000\089\000\218\000\057\000\037\000\067\000\007\000\
\064\000\091\000\099\000\065\000\066\000\100\000\227\000\228\000\
\092\000\008\000\101\000\103\000\176\000\105\000\067\000\012\000\
\013\000\106\000\062\000\015\000\016\000\010\000\018\000\063\000\
\020\000\020\000\021\000\241\000\014\000\246\000\102\000\017\000\
\023\000\248\000\019\000\065\000\066\000\113\000\022\000\114\000\
\117\000\120\000\121\000\094\000\122\000\124\000\067\000\128\000\
\005\001\062\000\131\000\133\000\145\000\028\000\063\000\153\000\
\148\000\154\000\028\000\028\000\152\000\155\000\028\000\064\000\
\028\000\028\000\065\000\066\000\158\000\028\000\162\000\165\000\
\178\000\028\000\182\000\175\000\186\000\067\000\028\000\189\000\
\062\000\028\000\028\000\028\000\185\000\063\000\188\000\190\000\
\018\000\164\000\192\000\193\000\028\000\194\000\064\000\028\000\
\028\000\065\000\066\000\028\000\197\000\028\000\028\000\028\000\
\028\000\028\000\198\000\205\000\067\000\028\000\202\000\027\000\
\028\000\214\000\028\000\028\000\027\000\027\000\207\000\028\000\
\027\000\028\000\027\000\027\000\028\000\209\000\071\000\027\000\
\028\000\215\000\213\000\027\000\071\000\217\000\219\000\223\000\
\027\000\225\000\062\000\027\000\027\000\027\000\226\000\063\000\
\229\000\230\000\019\000\238\000\232\000\234\000\027\000\235\000\
\064\000\027\000\027\000\065\000\066\000\027\000\239\000\027\000\
\027\000\027\000\027\000\027\000\237\000\240\000\067\000\027\000\
\254\000\039\000\027\000\255\000\027\000\027\000\039\000\039\000\
\002\001\027\000\039\000\027\000\039\000\039\000\027\000\007\001\
\051\000\039\000\027\000\071\000\236\000\039\000\051\000\051\000\
\071\000\051\000\039\000\065\000\051\000\039\000\039\000\039\000\
\065\000\053\000\079\000\041\000\010\000\079\000\071\000\043\000\
\039\000\051\000\051\000\039\000\039\000\073\000\051\000\039\000\
\231\000\039\000\039\000\039\000\039\000\039\000\051\000\029\000\
\157\000\039\000\201\000\021\000\039\000\087\000\039\000\039\000\
\021\000\021\000\058\000\039\000\021\000\039\000\021\000\021\000\
\039\000\253\000\000\000\000\000\039\000\000\000\062\000\021\000\
\156\000\000\000\000\000\063\000\021\000\000\000\062\000\000\000\
\000\000\021\000\000\000\063\000\064\000\000\000\009\000\065\000\
\066\000\000\000\021\000\000\000\064\000\021\000\021\000\065\000\
\066\000\021\000\067\000\021\000\021\000\021\000\021\000\021\000\
\000\000\224\000\067\000\021\000\000\000\020\000\021\000\000\000\
\021\000\021\000\020\000\020\000\000\000\021\000\020\000\021\000\
\020\000\020\000\021\000\000\000\000\000\000\000\021\000\000\000\
\062\000\020\000\000\000\000\000\000\000\063\000\020\000\000\000\
\062\000\160\000\000\000\020\000\000\000\063\000\064\000\000\000\
\011\000\065\000\066\000\000\000\000\000\000\000\064\000\020\000\
\020\000\065\000\066\000\020\000\067\000\020\000\020\000\020\000\
\020\000\020\000\000\000\000\000\067\000\020\000\000\000\000\000\
\020\000\000\000\020\000\020\000\000\000\018\000\000\000\020\000\
\000\000\020\000\018\000\018\000\020\000\074\000\018\000\000\000\
\020\000\018\000\075\000\076\000\000\000\000\000\077\000\000\000\
\062\000\018\000\000\000\000\000\000\000\063\000\018\000\000\000\
\062\000\183\000\012\000\018\000\000\000\063\000\064\000\000\000\
\000\000\065\000\066\000\000\000\000\000\000\000\064\000\018\000\
\018\000\065\000\066\000\018\000\067\000\018\000\018\000\018\000\
\018\000\018\000\000\000\000\000\067\000\018\000\000\000\019\000\
\018\000\000\000\018\000\018\000\019\000\019\000\000\000\018\000\
\019\000\018\000\000\000\019\000\018\000\000\000\000\000\000\000\
\018\000\000\000\000\000\019\000\000\000\000\000\000\000\134\000\
\019\000\135\000\000\000\136\000\017\000\019\000\137\000\138\000\
\000\000\139\000\000\000\000\000\000\000\000\000\000\000\140\000\
\141\000\019\000\019\000\000\000\142\000\019\000\000\000\019\000\
\019\000\019\000\019\000\019\000\077\000\000\000\062\000\019\000\
\000\000\010\000\019\000\063\000\019\000\019\000\010\000\010\000\
\000\000\019\000\010\000\019\000\064\000\010\000\019\000\065\000\
\066\000\000\000\019\000\000\000\000\000\010\000\000\000\000\000\
\000\000\000\000\067\000\000\000\062\000\203\000\013\000\010\000\
\000\000\063\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\064\000\010\000\010\000\065\000\066\000\010\000\
\000\000\010\000\010\000\010\000\010\000\010\000\000\000\000\000\
\067\000\010\000\000\000\009\000\010\000\000\000\010\000\010\000\
\009\000\009\000\000\000\010\000\009\000\010\000\000\000\009\000\
\010\000\000\000\000\000\000\000\010\000\000\000\000\000\009\000\
\000\000\000\000\000\000\000\000\000\000\000\000\062\000\220\000\
\015\000\009\000\000\000\063\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\064\000\009\000\009\000\065\000\
\066\000\009\000\000\000\009\000\009\000\009\000\009\000\009\000\
\000\000\000\000\067\000\009\000\000\000\011\000\009\000\000\000\
\009\000\009\000\011\000\011\000\000\000\009\000\011\000\009\000\
\000\000\011\000\009\000\009\000\010\000\011\000\009\000\000\000\
\000\000\011\000\000\000\014\000\000\000\000\000\017\000\000\000\
\000\000\019\000\014\000\011\000\000\000\022\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\011\000\
\011\000\000\000\000\000\011\000\000\000\011\000\011\000\011\000\
\011\000\011\000\000\000\000\000\062\000\011\000\000\000\012\000\
\011\000\063\000\011\000\011\000\012\000\012\000\000\000\011\000\
\012\000\011\000\064\000\012\000\011\000\065\000\066\000\000\000\
\011\000\000\000\000\000\012\000\000\000\000\000\000\000\000\000\
\067\000\000\000\062\000\222\000\148\000\012\000\000\000\063\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\064\000\012\000\012\000\065\000\066\000\012\000\000\000\012\000\
\012\000\012\000\012\000\012\000\000\000\000\000\067\000\012\000\
\000\000\017\000\012\000\000\000\012\000\012\000\017\000\017\000\
\000\000\012\000\017\000\012\000\000\000\017\000\012\000\000\000\
\000\000\000\000\012\000\000\000\000\000\017\000\000\000\000\000\
\000\000\000\000\000\000\000\000\062\000\249\000\000\000\017\000\
\000\000\063\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\064\000\017\000\017\000\065\000\066\000\017\000\
\000\000\017\000\017\000\017\000\017\000\017\000\000\000\000\000\
\067\000\017\000\000\000\013\000\017\000\000\000\017\000\017\000\
\013\000\013\000\000\000\017\000\013\000\017\000\000\000\013\000\
\017\000\000\000\000\000\000\000\017\000\000\000\000\000\013\000\
\000\000\000\000\000\000\000\000\000\000\000\000\062\000\000\001\
\000\000\013\000\000\000\063\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\064\000\013\000\013\000\065\000\
\066\000\013\000\000\000\013\000\013\000\013\000\013\000\013\000\
\000\000\000\000\067\000\013\000\000\000\015\000\013\000\000\000\
\013\000\013\000\015\000\015\000\000\000\013\000\015\000\013\000\
\000\000\015\000\013\000\000\000\000\000\000\000\013\000\000\000\
\000\000\015\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\015\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\015\000\
\015\000\000\000\000\000\015\000\000\000\015\000\015\000\015\000\
\015\000\015\000\000\000\000\000\000\000\015\000\000\000\014\000\
\015\000\000\000\015\000\015\000\014\000\014\000\000\000\015\000\
\014\000\015\000\000\000\014\000\015\000\000\000\000\000\000\000\
\015\000\000\000\000\000\014\000\000\000\000\000\000\000\000\000\
\000\000\003\000\004\000\005\000\000\000\014\000\000\000\000\000\
\006\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\000\000\014\000\014\000\000\000\000\000\014\000\000\000\014\000\
\014\000\014\000\014\000\014\000\000\000\000\000\000\000\014\000\
\000\000\000\000\014\000\000\000\014\000\014\000\006\000\006\000\
\006\000\014\000\007\000\014\000\000\000\006\000\014\000\000\000\
\000\000\000\000\014\000\000\000\008\000\009\000\010\000\011\000\
\000\000\000\000\012\000\013\000\000\000\014\000\015\000\016\000\
\017\000\018\000\000\000\019\000\020\000\021\000\000\000\022\000\
\000\000\000\000\000\000\023\000\000\000\024\000\000\000\006\000\
\007\000\007\000\007\000\000\000\000\000\000\000\000\000\007\000\
\000\000\006\000\006\000\006\000\006\000\000\000\000\000\006\000\
\006\000\000\000\006\000\006\000\006\000\006\000\006\000\000\000\
\006\000\006\000\006\000\000\000\006\000\000\000\000\000\006\000\
\006\000\000\000\000\000\003\000\004\000\005\000\000\000\000\000\
\000\000\007\000\006\000\000\000\000\000\000\000\000\000\000\000\
\000\000\000\000\000\000\007\000\007\000\007\000\007\000\000\000\
\000\000\007\000\007\000\000\000\007\000\007\000\007\000\007\000\
\007\000\000\000\007\000\007\000\007\000\000\000\007\000\000\000\
\000\000\007\000\007\000\000\000\007\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\008\000\009\000\
\010\000\011\000\000\000\000\000\012\000\013\000\000\000\014\000\
\015\000\016\000\017\000\018\000\000\000\019\000\020\000\021\000\
\000\000\022\000\062\000\000\000\000\000\023\000\000\000\063\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\064\000\000\000\000\000\065\000\066\000\000\000\000\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\067\000\000\000\
\000\000\000\000\000\000\000\000\000\000\000\000\000\000\009\000\
\010\000\011\000\000\000\000\000\000\000\000\000\000\000\014\000\
\000\000\000\000\017\000\000\000\000\000\019\000\000\000\000\000\
\000\000\022\000"

let yycheck = "\040\000\
\000\000\001\001\001\000\046\000\000\000\122\000\049\000\006\000\
\008\001\008\000\059\000\015\000\016\000\010\001\131\000\022\001\
\040\000\029\000\009\000\018\000\010\001\033\000\021\000\001\001\
\002\001\003\001\016\001\051\000\025\001\028\000\008\001\134\000\
\135\000\136\000\137\000\138\000\139\000\140\000\141\000\142\000\
\047\001\158\000\145\000\022\001\033\000\086\000\025\001\010\001\
\039\000\001\001\010\001\027\000\028\000\016\001\017\001\031\000\
\105\000\017\001\000\000\071\000\060\001\022\001\014\001\062\000\
\063\000\064\000\065\000\066\000\067\000\022\001\001\000\095\000\
\025\001\001\001\002\001\003\001\014\001\076\000\077\000\243\000\
\008\001\059\001\071\000\082\000\022\001\048\001\047\001\251\000\
\048\001\088\000\193\000\194\000\091\000\092\000\070\000\094\000\
\100\000\075\001\070\001\098\000\124\000\001\001\101\000\001\001\
\103\000\021\001\223\000\010\001\008\001\001\001\022\001\014\001\
\001\001\001\001\042\001\001\001\000\000\033\001\034\001\008\001\
\237\000\014\001\025\001\035\001\052\001\053\001\054\001\055\001\
\044\001\022\001\058\001\059\001\035\001\061\001\062\001\063\001\
\064\001\065\001\179\000\067\001\068\001\069\001\035\001\071\001\
\047\001\021\001\074\001\075\001\014\001\079\001\078\001\001\001\
\002\001\152\000\047\001\254\000\022\001\033\001\034\001\200\000\
\003\001\074\001\205\000\008\001\001\001\164\000\001\001\002\001\
\003\001\035\001\016\001\008\001\001\001\008\001\000\000\021\001\
\175\000\176\000\016\001\008\001\001\001\047\001\014\001\021\001\
\030\001\025\001\185\000\033\001\034\001\188\000\001\001\190\000\
\030\001\001\001\016\001\033\001\034\001\008\001\044\001\021\001\
\008\001\242\000\022\001\202\000\001\001\002\001\044\001\042\001\
\030\001\014\001\001\001\033\001\034\001\001\001\213\000\214\000\
\035\001\052\001\066\001\022\001\066\001\078\001\044\001\058\001\
\059\001\001\001\016\001\062\001\063\001\054\001\065\001\021\001\
\000\000\068\001\069\001\234\000\061\001\236\000\076\001\064\001\
\075\001\240\000\067\001\033\001\034\001\000\000\071\001\074\001\
\001\001\010\001\001\001\073\001\022\001\010\001\044\001\047\001\
\255\000\016\001\022\001\022\001\022\001\005\001\021\001\001\001\
\048\001\080\001\010\001\011\001\022\001\010\001\014\001\030\001\
\016\001\017\001\033\001\034\001\022\001\021\001\072\001\001\001\
\047\001\025\001\057\001\025\001\000\000\044\001\030\001\001\001\
\016\001\033\001\034\001\035\001\022\001\021\001\025\001\047\001\
\000\000\056\001\025\001\010\001\044\001\010\001\030\001\047\001\
\048\001\033\001\034\001\051\001\010\001\053\001\054\001\055\001\
\056\001\057\001\072\001\010\001\044\001\061\001\035\001\005\001\
\064\001\025\001\066\001\067\001\010\001\011\001\051\001\071\001\
\014\001\073\001\016\001\017\001\076\001\048\001\074\001\021\001\
\080\001\001\001\066\001\025\001\080\001\010\001\000\000\022\001\
\030\001\001\001\016\001\033\001\034\001\035\001\017\001\021\001\
\051\001\001\001\000\000\010\001\080\001\025\001\044\001\022\001\
\030\001\047\001\048\001\033\001\034\001\051\001\000\000\053\001\
\054\001\055\001\056\001\057\001\022\001\025\001\044\001\061\001\
\016\001\005\001\064\001\025\001\066\001\067\001\010\001\011\001\
\010\001\071\001\014\001\073\001\016\001\017\001\076\001\010\001\
\010\001\021\001\080\001\000\000\066\001\025\001\016\001\017\001\
\074\001\022\001\030\001\047\001\022\001\033\001\034\001\035\001\
\022\001\022\001\025\001\010\001\000\000\010\001\080\001\057\001\
\044\001\035\001\035\001\047\001\048\001\017\001\010\001\051\001\
\218\000\053\001\054\001\055\001\056\001\057\001\048\001\066\001\
\119\000\061\001\180\000\005\001\064\001\041\000\066\001\067\001\
\010\001\011\001\022\000\071\001\014\001\073\001\016\001\017\001\
\076\001\244\000\255\255\255\255\080\001\255\255\016\001\025\001\
\010\001\255\255\255\255\021\001\030\001\255\255\016\001\255\255\
\255\255\035\001\255\255\021\001\030\001\255\255\000\000\033\001\
\034\001\255\255\044\001\255\255\030\001\047\001\048\001\033\001\
\034\001\051\001\044\001\053\001\054\001\055\001\056\001\057\001\
\255\255\051\001\044\001\061\001\255\255\005\001\064\001\255\255\
\066\001\067\001\010\001\011\001\255\255\071\001\014\001\073\001\
\016\001\017\001\076\001\255\255\255\255\255\255\080\001\255\255\
\016\001\025\001\255\255\255\255\255\255\021\001\030\001\255\255\
\016\001\017\001\255\255\035\001\255\255\021\001\030\001\255\255\
\000\000\033\001\034\001\255\255\255\255\255\255\030\001\047\001\
\048\001\033\001\034\001\051\001\044\001\053\001\054\001\055\001\
\056\001\057\001\255\255\255\255\044\001\061\001\255\255\255\255\
\064\001\255\255\066\001\067\001\255\255\005\001\255\255\071\001\
\255\255\073\001\010\001\011\001\076\001\005\001\014\001\255\255\
\080\001\017\001\010\001\011\001\255\255\255\255\014\001\255\255\
\016\001\025\001\255\255\255\255\255\255\021\001\030\001\255\255\
\016\001\017\001\000\000\035\001\255\255\021\001\030\001\255\255\
\255\255\033\001\034\001\255\255\255\255\255\255\030\001\047\001\
\048\001\033\001\034\001\051\001\044\001\053\001\054\001\055\001\
\056\001\057\001\255\255\255\255\044\001\061\001\255\255\005\001\
\064\001\255\255\066\001\067\001\010\001\011\001\255\255\071\001\
\014\001\073\001\255\255\017\001\076\001\255\255\255\255\255\255\
\080\001\255\255\255\255\025\001\255\255\255\255\255\255\001\001\
\030\001\003\001\255\255\005\001\000\000\035\001\008\001\009\001\
\255\255\011\001\255\255\255\255\255\255\255\255\255\255\017\001\
\018\001\047\001\048\001\255\255\022\001\051\001\255\255\053\001\
\054\001\055\001\056\001\057\001\014\001\255\255\016\001\061\001\
\255\255\005\001\064\001\021\001\066\001\067\001\010\001\011\001\
\255\255\071\001\014\001\073\001\030\001\017\001\076\001\033\001\
\034\001\255\255\080\001\255\255\255\255\025\001\255\255\255\255\
\255\255\255\255\044\001\255\255\016\001\017\001\000\000\035\001\
\255\255\021\001\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\030\001\047\001\048\001\033\001\034\001\051\001\
\255\255\053\001\054\001\055\001\056\001\057\001\255\255\255\255\
\044\001\061\001\255\255\005\001\064\001\255\255\066\001\067\001\
\010\001\011\001\255\255\071\001\014\001\073\001\255\255\017\001\
\076\001\255\255\255\255\255\255\080\001\255\255\255\255\025\001\
\255\255\255\255\255\255\255\255\255\255\255\255\016\001\017\001\
\000\000\035\001\255\255\021\001\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\030\001\047\001\048\001\033\001\
\034\001\051\001\255\255\053\001\054\001\055\001\056\001\057\001\
\255\255\255\255\044\001\061\001\255\255\005\001\064\001\255\255\
\066\001\067\001\010\001\011\001\255\255\071\001\014\001\073\001\
\255\255\017\001\076\001\053\001\054\001\055\001\080\001\255\255\
\255\255\025\001\255\255\061\001\255\255\255\255\064\001\255\255\
\255\255\067\001\000\000\035\001\255\255\071\001\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\047\001\
\048\001\255\255\255\255\051\001\255\255\053\001\054\001\055\001\
\056\001\057\001\255\255\255\255\016\001\061\001\255\255\005\001\
\064\001\021\001\066\001\067\001\010\001\011\001\255\255\071\001\
\014\001\073\001\030\001\017\001\076\001\033\001\034\001\255\255\
\080\001\255\255\255\255\025\001\255\255\255\255\255\255\255\255\
\044\001\255\255\016\001\017\001\048\001\035\001\255\255\021\001\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\030\001\047\001\048\001\033\001\034\001\051\001\255\255\053\001\
\054\001\055\001\056\001\057\001\255\255\255\255\044\001\061\001\
\255\255\005\001\064\001\255\255\066\001\067\001\010\001\011\001\
\255\255\071\001\014\001\073\001\255\255\017\001\076\001\255\255\
\255\255\255\255\080\001\255\255\255\255\025\001\255\255\255\255\
\255\255\255\255\255\255\255\255\016\001\017\001\255\255\035\001\
\255\255\021\001\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\030\001\047\001\048\001\033\001\034\001\051\001\
\255\255\053\001\054\001\055\001\056\001\057\001\255\255\255\255\
\044\001\061\001\255\255\005\001\064\001\255\255\066\001\067\001\
\010\001\011\001\255\255\071\001\014\001\073\001\255\255\017\001\
\076\001\255\255\255\255\255\255\080\001\255\255\255\255\025\001\
\255\255\255\255\255\255\255\255\255\255\255\255\016\001\017\001\
\255\255\035\001\255\255\021\001\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\030\001\047\001\048\001\033\001\
\034\001\051\001\255\255\053\001\054\001\055\001\056\001\057\001\
\255\255\255\255\044\001\061\001\255\255\005\001\064\001\255\255\
\066\001\067\001\010\001\011\001\255\255\071\001\014\001\073\001\
\255\255\017\001\076\001\255\255\255\255\255\255\080\001\255\255\
\255\255\025\001\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\035\001\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\047\001\
\048\001\255\255\255\255\051\001\255\255\053\001\054\001\055\001\
\056\001\057\001\255\255\255\255\255\255\061\001\255\255\005\001\
\064\001\255\255\066\001\067\001\010\001\011\001\255\255\071\001\
\014\001\073\001\255\255\017\001\076\001\255\255\255\255\255\255\
\080\001\255\255\255\255\025\001\255\255\255\255\255\255\255\255\
\255\255\001\001\002\001\003\001\255\255\035\001\255\255\255\255\
\008\001\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\255\255\047\001\048\001\255\255\255\255\051\001\255\255\053\001\
\054\001\055\001\056\001\057\001\255\255\255\255\255\255\061\001\
\255\255\255\255\064\001\255\255\066\001\067\001\001\001\002\001\
\003\001\071\001\042\001\073\001\255\255\008\001\076\001\255\255\
\255\255\255\255\080\001\255\255\052\001\053\001\054\001\055\001\
\255\255\255\255\058\001\059\001\255\255\061\001\062\001\063\001\
\064\001\065\001\255\255\067\001\068\001\069\001\255\255\071\001\
\255\255\255\255\255\255\075\001\255\255\077\001\255\255\042\001\
\001\001\002\001\003\001\255\255\255\255\255\255\255\255\008\001\
\255\255\052\001\053\001\054\001\055\001\255\255\255\255\058\001\
\059\001\255\255\061\001\062\001\063\001\064\001\065\001\255\255\
\067\001\068\001\069\001\255\255\071\001\255\255\255\255\074\001\
\075\001\255\255\255\255\001\001\002\001\003\001\255\255\255\255\
\255\255\042\001\008\001\255\255\255\255\255\255\255\255\255\255\
\255\255\255\255\255\255\052\001\053\001\054\001\055\001\255\255\
\255\255\058\001\059\001\255\255\061\001\062\001\063\001\064\001\
\065\001\255\255\067\001\068\001\069\001\255\255\071\001\255\255\
\255\255\074\001\075\001\255\255\042\001\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\052\001\053\001\
\054\001\055\001\255\255\255\255\058\001\059\001\255\255\061\001\
\062\001\063\001\064\001\065\001\255\255\067\001\068\001\069\001\
\255\255\071\001\016\001\255\255\255\255\075\001\255\255\021\001\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
\030\001\255\255\255\255\033\001\034\001\255\255\255\255\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\044\001\255\255\
\255\255\255\255\255\255\255\255\255\255\255\255\255\255\053\001\
\054\001\055\001\255\255\255\255\255\255\255\255\255\255\061\001\
\255\255\255\255\064\001\255\255\255\255\067\001\255\255\255\255\
\255\255\071\001"

let yynames_const = "\
  BANG_\000\
  PERCENT_\000\
  AMPER_\000\
  AMPER_AMPER_\000\
  LPAREN_\000\
  LPAREN_RPAREN_\000\
  RPAREN_\000\
  STAR_\000\
  PLUS_\000\
  PLUS_PLUS_\000\
  COMMA_\000\
  DASH_\000\
  DASH_GT_\000\
  PERIOD_\000\
  PERIOD_LPAREN_\000\
  PERIOD_PERIOD_\000\
  SLASH_\000\
  SLASH_BACKSL_\000\
  COLON_\000\
  COLON_COLON_\000\
  COLON_LT_\000\
  COLON_EQ_\000\
  COLON_GT_\000\
  SEMI_\000\
  LT_\000\
  LT_DASH_\000\
  LT_DASH_GT_\000\
  LT_COLON_\000\
  LT_EQ_\000\
  LT_GT_\000\
  EQ_\000\
  EQ_GT_\000\
  EQ_UNDER_D_\000\
  GT_\000\
  GT_DASH_GT_\000\
  GT_EQ_\000\
  QUEST_\000\
  QUEST_EQ_\000\
  AROBAS_\000\
  LBRACK_\000\
  BACKSL_SLASH_\000\
  RBRACK_\000\
  HAT_\000\
  LBRACE_\000\
  BAR_\000\
  BAR_DASH_\000\
  BAR_BAR_\000\
  RBRACE_\000\
  TILDE_\000\
  MUSTUSE\000\
  DEFINITION\000\
  DEPENDS\000\
  ELSE\000\
  END\000\
  EXISTS\000\
  FALSE\000\
  FIX\000\
  FIXPOINT\000\
  FORALL\000\
  FUN\000\
  FUNCTION\000\
  IF\000\
  IN\000\
  INDUCTIVE\000\
  LET\000\
  MATCH\000\
  ON\000\
  PARAMETER\000\
  STRUCT\000\
  THEN\000\
  THEOREM\000\
  TRUE\000\
  WITH\000\
  BEGINPROOF\000\
  BEGINHEADER\000\
  ENDPROOF\000\
  EOF\000\
  "

let yynames_block = "\
  IDENT\000\
  STRING\000\
  NUM\000\
  BEGINNAME\000\
  "

let yyact = [|
  (fun _ -> failwith "parser")
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 6 : 'hyp_def_list) in
    let _3 = (Parsing.peek_val __caml_parser_env 4 : string) in
    let _5 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    Obj.repr(
# 215 "parsecoq.mly"
      ( (_3, (Hyp (goal_name, enot _5, 0), false) :: _1) )
# 861 "parsecoq.ml"
               : string * (Phrase.phrase * bool) list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 8 : 'proof_head) in
    let _2 = (Parsing.peek_val __caml_parser_env 7 : 'hyp_def_list) in
    let _4 = (Parsing.peek_val __caml_parser_env 5 : string) in
    let _6 = (Parsing.peek_val __caml_parser_env 3 : 'expr) in
    Obj.repr(
# 217 "parsecoq.mly"
      ( (_4, (Hyp (goal_name, enot _6, 0), false) :: _2) )
# 871 "parsecoq.ml"
               : string * (Phrase.phrase * bool) list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'hyp_def_list) in
    Obj.repr(
# 219 "parsecoq.mly"
      ( (* Error.warn "deprecated input format"; *)
        (thm_default_name, (Hyp (goal_name, enot _1, 0), false) :: _2) )
# 880 "parsecoq.ml"
               : string * (Phrase.phrase * bool) list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 4 : 'proof_head) in
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 2 : 'hyp_def_list) in
    Obj.repr(
# 222 "parsecoq.mly"
      ( (* Error.warn "deprecated input format"; *)
        (_1, (Hyp (goal_name, enot _2, 0), false) :: _3) )
# 890 "parsecoq.ml"
               : string * (Phrase.phrase * bool) list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'proofheaders) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'proofheaders) in
    Obj.repr(
# 228 "parsecoq.mly"
      ( _3 )
# 899 "parsecoq.ml"
               : 'proof_head))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'proofheaders) in
    Obj.repr(
# 230 "parsecoq.mly"
      ( "theorem" )
# 906 "parsecoq.ml"
               : 'proof_head))
; (fun __caml_parser_env ->
    Obj.repr(
# 235 "parsecoq.mly"
      ( () )
# 912 "parsecoq.ml"
               : 'proofheaders))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'proofheaders) in
    Obj.repr(
# 237 "parsecoq.mly"
      ( () )
# 919 "parsecoq.ml"
               : 'proofheaders))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'bindings) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 242 "parsecoq.mly"
      ( mk_all _2 _4 )
# 927 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'bindings) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 244 "parsecoq.mly"
      ( mk_ex _2 _4 )
# 935 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'bindings) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 247 "parsecoq.mly"
      ( mk_lam _2 _4 )
# 943 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'fix) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 250 "parsecoq.mly"
      ( mk_let_fix _2 _4 )
# 951 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 253 "parsecoq.mly"
      ( mk_let _2 _4 _6 )
# 960 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 7 : string) in
    let _5 = (Parsing.peek_val __caml_parser_env 5 : 'junk) in
    let _8 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _10 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 256 "parsecoq.mly"
      ( mk_let _3 _8 _10 )
# 970 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 6 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 4 : 'junk) in
    let _6 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _8 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 259 "parsecoq.mly"
      ( mk_let _2 _6 _8 )
# 980 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'expr) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'pat_expr_list) in
    Obj.repr(
# 262 "parsecoq.mly"
      ( eapp ("$match", _2 :: _4) )
# 988 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : 'expr) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 265 "parsecoq.mly"
      ( eapp ("FOCAL.ifthenelse", [_2; _4; _6]) )
# 997 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 268 "parsecoq.mly"
      ( eimply (_1, _3) )
# 1005 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 271 "parsecoq.mly"
      ( eequiv (_1, _3) )
# 1013 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 274 "parsecoq.mly"
      ( eor (_1, _3) )
# 1021 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 277 "parsecoq.mly"
      ( eand (_1, _3) )
# 1029 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 280 "parsecoq.mly"
      ( eapp ("=", [_1; _3]) )
# 1037 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 282 "parsecoq.mly"
      ( enot (eapp ("=", [_1; _3])) )
# 1045 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 285 "parsecoq.mly"
      ( enot (_2) )
# 1052 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'expr1) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'expr1_list) in
    Obj.repr(
# 288 "parsecoq.mly"
      ( mk_apply (_1, _2) )
# 1060 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'expr1_list) in
    Obj.repr(
# 291 "parsecoq.mly"
      ( mk_eapp ("@", evar (_2) :: _3) )
# 1068 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 294 "parsecoq.mly"
      ( mk_eapp ("@", [evar (_2)]) )
# 1075 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'expr1) in
    Obj.repr(
# 297 "parsecoq.mly"
      ( _1 )
# 1082 "parsecoq.ml"
               : 'expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 9 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 8 : 'bindings) in
    let _6 = (Parsing.peek_val __caml_parser_env 5 : string) in
    let _9 = (Parsing.peek_val __caml_parser_env 2 : 'typ) in
    let _11 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 302 "parsecoq.mly"
      ( mk_fix _2 _9 _3 _11 )
# 1093 "parsecoq.ml"
               : 'fix))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 307 "parsecoq.mly"
      ( evar (_1) )
# 1100 "parsecoq.ml"
               : 'expr1))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 309 "parsecoq.mly"
      ( eapp (_1, []) )
# 1107 "parsecoq.ml"
               : 'expr1))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 313 "parsecoq.mly"
      ( eapp ("$string", [mk_string _1]) )
# 1114 "parsecoq.ml"
               : 'expr1))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 1 : 'comma_expr_list) in
    Obj.repr(
# 315 "parsecoq.mly"
      ( mk_pairs _2 _3 )
# 1122 "parsecoq.ml"
               : 'expr1))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'expr) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    Obj.repr(
# 317 "parsecoq.mly"
      ( eapp ("*", [_2; _4]) )
# 1130 "parsecoq.ml"
               : 'expr1))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'expr) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : string) in
    Obj.repr(
# 319 "parsecoq.mly"
      ( eapp ("%", [_2; evar (_4)]) )
# 1138 "parsecoq.ml"
               : 'expr1))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    Obj.repr(
# 321 "parsecoq.mly"
      ( _2 )
# 1145 "parsecoq.ml"
               : 'expr1))
; (fun __caml_parser_env ->
    Obj.repr(
# 323 "parsecoq.mly"
      ( etrue )
# 1151 "parsecoq.ml"
               : 'expr1))
; (fun __caml_parser_env ->
    Obj.repr(
# 325 "parsecoq.mly"
      ( efalse )
# 1157 "parsecoq.ml"
               : 'expr1))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'expr1) in
    Obj.repr(
# 329 "parsecoq.mly"
                           ( [_1] )
# 1164 "parsecoq.ml"
               : 'expr1_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'expr1) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'expr1_list) in
    Obj.repr(
# 330 "parsecoq.mly"
                           ( _1 :: _2 )
# 1172 "parsecoq.ml"
               : 'expr1_list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 335 "parsecoq.mly"
      ( [_2] )
# 1179 "parsecoq.ml"
               : 'comma_expr_list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'comma_expr_list) in
    Obj.repr(
# 337 "parsecoq.mly"
      ( _2 :: _3 )
# 1187 "parsecoq.ml"
               : 'comma_expr_list))
; (fun __caml_parser_env ->
    Obj.repr(
# 342 "parsecoq.mly"
      ( [] )
# 1193 "parsecoq.ml"
               : 'pat_expr_list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'pattern) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'pat_expr_list) in
    Obj.repr(
# 344 "parsecoq.mly"
      ( mk_pattern _2 _4 :: _5 )
# 1202 "parsecoq.ml"
               : 'pat_expr_list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'pattern) in
    Obj.repr(
# 349 "parsecoq.mly"
      ( _2 )
# 1209 "parsecoq.ml"
               : 'pattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'idlist) in
    Obj.repr(
# 351 "parsecoq.mly"
      ( (_1, _2) )
# 1217 "parsecoq.ml"
               : 'pattern))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : string) in
    Obj.repr(
# 353 "parsecoq.mly"
      ( ("Datatypes.pair", [_2; _4]) )
# 1225 "parsecoq.ml"
               : 'pattern))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'simplebinding) in
    Obj.repr(
# 357 "parsecoq.mly"
                           ( _1 )
# 1232 "parsecoq.ml"
               : 'bindings))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'binding_list) in
    Obj.repr(
# 358 "parsecoq.mly"
                           ( _1 )
# 1239 "parsecoq.ml"
               : 'bindings))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'idlist) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'typ) in
    Obj.repr(
# 362 "parsecoq.mly"
                       ( List.map (fun v -> (v, _3)) _1 )
# 1247 "parsecoq.ml"
               : 'simplebinding))
; (fun __caml_parser_env ->
    Obj.repr(
# 366 "parsecoq.mly"
                           ( [] )
# 1253 "parsecoq.ml"
               : 'idlist))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'idlist) in
    Obj.repr(
# 367 "parsecoq.mly"
                           ( _1 :: _2 )
# 1261 "parsecoq.ml"
               : 'idlist))
; (fun __caml_parser_env ->
    Obj.repr(
# 372 "parsecoq.mly"
      ( [] )
# 1267 "parsecoq.ml"
               : 'binding_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'binding_list) in
    Obj.repr(
# 374 "parsecoq.mly"
      ( (_1, "") :: _2 )
# 1275 "parsecoq.ml"
               : 'binding_list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'simplebinding) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'binding_list) in
    Obj.repr(
# 376 "parsecoq.mly"
      ( _2 @ _4 )
# 1283 "parsecoq.ml"
               : 'binding_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'expr) in
    Obj.repr(
# 380 "parsecoq.mly"
                           ( mk_type_string _1 )
# 1290 "parsecoq.ml"
               : 'typ))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 385 "parsecoq.mly"
           ( _1 )
# 1297 "parsecoq.ml"
               : 'id_or_expr))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : string) in
    Obj.repr(
# 386 "parsecoq.mly"
           ( _1 )
# 1304 "parsecoq.ml"
               : 'id_or_expr))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'id_or_expr) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    Obj.repr(
# 392 "parsecoq.mly"
      ( Hyp (_2, _4, 1) )
# 1312 "parsecoq.ml"
               : 'hyp_def))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'id_or_expr) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    Obj.repr(
# 394 "parsecoq.mly"
      ( let (params, expr) = get_params _4 in
        Def (DefReal (_2, _2, params, expr, None)) )
# 1321 "parsecoq.ml"
               : 'hyp_def))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 6 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 5 : 'compact_args) in
    let _5 = (Parsing.peek_val __caml_parser_env 3 : 'typ) in
    let _7 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    Obj.repr(
# 397 "parsecoq.mly"
      (
       let compact_params = _3 in
       let (other_params, expr) = get_params _7 in
       Def (DefReal (_2, _2, (compact_params @ other_params), expr, None))
      )
# 1335 "parsecoq.ml"
               : 'hyp_def))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 10 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 9 : 'compact_args) in
    let _6 = (Parsing.peek_val __caml_parser_env 6 : string) in
    let _9 = (Parsing.peek_val __caml_parser_env 3 : 'typ) in
    let _11 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    Obj.repr(
# 404 "parsecoq.mly"
      (
       let compact_params = _3 in
       let (other_params, expr) = get_params _11 in
       Def (DefReal (_2, _2, (compact_params @ other_params), expr, Some _6))
      )
# 1350 "parsecoq.ml"
               : 'hyp_def))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 9 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 8 : 'compact_args) in
    let _5 = (Parsing.peek_val __caml_parser_env 6 : 'typ) in
    let _7 = (Parsing.peek_val __caml_parser_env 4 : 'expr) in
    let _10 = (Parsing.peek_val __caml_parser_env 1 : 'expr) in
    Obj.repr(
# 411 "parsecoq.mly"
      ( Def (DefRec (_7, _2, _3, _10)) )
# 1361 "parsecoq.ml"
               : 'hyp_def))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 6 : string) in
    let _3 = (Parsing.peek_val __caml_parser_env 5 : 'binding_list) in
    let _5 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _7 = (Parsing.peek_val __caml_parser_env 1 : 'constr_list) in
    Obj.repr(
# 413 "parsecoq.mly"
      ( mk_inductive _2 _3 _7 )
# 1371 "parsecoq.ml"
               : 'hyp_def))
; (fun __caml_parser_env ->
    Obj.repr(
# 418 "parsecoq.mly"
                                                         ( [] )
# 1377 "parsecoq.ml"
               : 'compact_args))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 4 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 2 : 'typ) in
    let _6 = (Parsing.peek_val __caml_parser_env 0 : 'compact_args) in
    Obj.repr(
# 419 "parsecoq.mly"
                                                     ( (evar _2) :: _6 )
# 1386 "parsecoq.ml"
               : 'compact_args))
; (fun __caml_parser_env ->
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'hyp_def) in
    Obj.repr(
# 424 "parsecoq.mly"
                               (
      (* FIXME activate this warning at some point.
      Error.warn "use of obsolete \"Depends on\" syntax";
      *)
      (_3, true)
    )
# 1398 "parsecoq.ml"
               : 'dep_hyp_def))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'hyp_def) in
    Obj.repr(
# 430 "parsecoq.mly"
                               ( (_2, true) )
# 1405 "parsecoq.ml"
               : 'dep_hyp_def))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'hyp_def) in
    Obj.repr(
# 431 "parsecoq.mly"
                               ( (_1, false) )
# 1412 "parsecoq.ml"
               : 'dep_hyp_def))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : 'dep_hyp_def) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'hyp_def_list) in
    Obj.repr(
# 435 "parsecoq.mly"
                               ( _1 :: _2 )
# 1420 "parsecoq.ml"
               : 'hyp_def_list))
; (fun __caml_parser_env ->
    Obj.repr(
# 436 "parsecoq.mly"
                               ( [] )
# 1426 "parsecoq.ml"
               : 'hyp_def_list))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : string) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'constr_type) in
    let _5 = (Parsing.peek_val __caml_parser_env 0 : 'constr_list) in
    Obj.repr(
# 441 "parsecoq.mly"
      ( (_2, _4) :: _5 )
# 1435 "parsecoq.ml"
               : 'constr_list))
; (fun __caml_parser_env ->
    Obj.repr(
# 443 "parsecoq.mly"
      ( [] )
# 1441 "parsecoq.ml"
               : 'constr_list))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 0 : 'arg_type) in
    Obj.repr(
# 447 "parsecoq.mly"
                                      ( [] )
# 1448 "parsecoq.ml"
               : 'constr_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 2 : 'arg_type) in
    let _3 = (Parsing.peek_val __caml_parser_env 0 : 'constr_type) in
    Obj.repr(
# 448 "parsecoq.mly"
                                      ( _1 :: _3 )
# 1456 "parsecoq.ml"
               : 'constr_type))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 3 : 'arg_type) in
    let _4 = (Parsing.peek_val __caml_parser_env 1 : 'constr_type) in
    Obj.repr(
# 450 "parsecoq.mly"
                                      ( _2 :: _4 )
# 1464 "parsecoq.ml"
               : 'constr_type))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 1 : 'arg_type) in
    Obj.repr(
# 454 "parsecoq.mly"
                                      ( _2 )
# 1471 "parsecoq.ml"
               : 'arg_type))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'idlist) in
    Obj.repr(
# 455 "parsecoq.mly"
                                      ( (_1, _2) )
# 1479 "parsecoq.ml"
               : 'arg_type))
; (fun __caml_parser_env ->
    Obj.repr(
# 459 "parsecoq.mly"
                                      ( () )
# 1485 "parsecoq.ml"
               : 'junk))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'junk) in
    Obj.repr(
# 460 "parsecoq.mly"
                                      ( () )
# 1493 "parsecoq.ml"
               : 'junk))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'junk) in
    Obj.repr(
# 461 "parsecoq.mly"
                                      ( () )
# 1500 "parsecoq.ml"
               : 'junk))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'junk) in
    Obj.repr(
# 462 "parsecoq.mly"
                                      ( () )
# 1507 "parsecoq.ml"
               : 'junk))
; (fun __caml_parser_env ->
    let _1 = (Parsing.peek_val __caml_parser_env 1 : string) in
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'junk) in
    Obj.repr(
# 463 "parsecoq.mly"
                                      ( () )
# 1515 "parsecoq.ml"
               : 'junk))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'junk) in
    Obj.repr(
# 464 "parsecoq.mly"
                                      ( () )
# 1522 "parsecoq.ml"
               : 'junk))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'junk) in
    Obj.repr(
# 465 "parsecoq.mly"
                                      ( () )
# 1529 "parsecoq.ml"
               : 'junk))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 0 : 'junk) in
    Obj.repr(
# 466 "parsecoq.mly"
                                      ( () )
# 1536 "parsecoq.ml"
               : 'junk))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'junk) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'junk) in
    Obj.repr(
# 467 "parsecoq.mly"
                                      ( () )
# 1544 "parsecoq.ml"
               : 'junk))
; (fun __caml_parser_env ->
    let _2 = (Parsing.peek_val __caml_parser_env 2 : 'junk) in
    let _4 = (Parsing.peek_val __caml_parser_env 0 : 'junk) in
    Obj.repr(
# 468 "parsecoq.mly"
                                      ( () )
# 1552 "parsecoq.ml"
               : 'junk))
(* Entry file *)
; (fun __caml_parser_env -> raise (Parsing.YYexit (Parsing.peek_val __caml_parser_env 0)))
|]
let yytables =
  { Parsing.actions=yyact;
    Parsing.transl_const=yytransl_const;
    Parsing.transl_block=yytransl_block;
    Parsing.lhs=yylhs;
    Parsing.len=yylen;
    Parsing.defred=yydefred;
    Parsing.dgoto=yydgoto;
    Parsing.sindex=yysindex;
    Parsing.rindex=yyrindex;
    Parsing.gindex=yygindex;
    Parsing.tablesize=yytablesize;
    Parsing.table=yytable;
    Parsing.check=yycheck;
    Parsing.error_function=parse_error;
    Parsing.names_const=yynames_const;
    Parsing.names_block=yynames_block }
let file (lexfun : Lexing.lexbuf -> token) (lexbuf : Lexing.lexbuf) =
   (Parsing.yyparse yytables 1 lexfun lexbuf : string * (Phrase.phrase * bool) list)
;;
